package bookExamples.ch26Graphics.draw2d;

import gui.In;
import gui.run.RunButton;
import gui.run.RunCheckBox;
import gui.run.RunSlider;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

public class Square2d extends DJShape implements PropertyEditor {
    int h = 1;
    int w = 1;
    int xc = 0;
    int yc = 0;

    String value;
    PropertyChangeSupport pcs = new PropertyChangeSupport(this);

    private Point2D scale = new Point2D.Double(1.0, 1.0);
    private Point2D shear = new Point2D.Double(0, 0);

    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new Square2d(x1, y1, x2, y2);
    }

    public int getX1() {
        return x1;
    }

    public int getY1() {
        return y1;
    }

    public int getW() {
        return w;
    }

    public int getH() {
        return h;
    }

    public Square2d(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        setX1(_x1);
        setY1(_y1);
        w = Math.abs(_x2 - x1);
        h = Math.abs(_y2 - y1);
        if (_x1 > _x2) setX1(_x2);
        if (_y1 > _y2) setY1(_y2);
        xc = x1 + w / 2;
        yc = y1 + h / 2;

    }

    public Point getCenter() {
        return new Point(x1 + w / 2, y1 + w / 2);
    }

    public void draw(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        Color saveColor = g2d.getColor();
        g2d.setColor(getForeground());
        Point pc = getCenter();
        xc = (int) pc.getX();
        yc = (int) pc.getY();
        g2d.setTransform(at);
        g2d.drawRect(x1, y1, w, w);
        g2d.fillOval(xc, yc, 2, 2);
        g2d.drawString("(" + xc + "," + yc + ")", xc + 3, yc + 3);
        g2d.setColor(saveColor);
        restoreState(g2d);
    }

    public boolean isPaintable() {
        return true;
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    public Component getCustomEditor() {
        return editorPanel();
    }

    private int translateX = 0;
    private int translateY = 0;
    private double scaleX = 1.0;
    private double scaleY = 1.0;
    private double shearX = 0.0;
    private double shearY = 0.0;
    double pi = Math.PI;
    RunCheckBox rcb = new RunCheckBox("get selected point") {
        public void run() {
            if (isSelected())
                rs.setEnabled(true);
            else if (!isSelected())
                rs.setEnabled(false);
        }
    };
    RunSlider rs = new RunSlider(RunSlider.HORIZONTAL, -180, 180, 0) {
        public void run() {
            pcs.firePropertyChange("square", this, null);
            at.rotate(getValue() * pi / 180,
                    getX1(), getY1());

        }
    };

    private Component editorPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());
        rs.setEnabled(false);
        jp.add(new JLabel("size"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 2000, w) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                w = getValue();
            }
        });
        jp.add(new JLabel("translateX"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 1000, w) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                //x1 = getValue();
                translateX = getValue();
                at.setToTranslation(translateX, translateY);
            }
        });
        jp.add(new JLabel("translateY"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 1000, w) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                //y1 = getValue();
                translateY = getValue();
                at.setToTranslation(translateX, translateY);
            }
        });
        jp.add(new JLabel("rotate about center"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, -180, 180, 0) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                at.rotate(getValue() * pi / 180,
                        getCenter().getX(), getCenter().getY());
            }
        });
        jp.add(new JLabel("rotate about point"));
        jp.add(rcb);
        jp.add(rs);
        jp.add(new JLabel("shearX"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 400, 200) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                shearX = getValue() / 100;
                at.shear(shearX, shearY);
            }
        });
        jp.add(new JLabel("shearY"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 400, 200) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                shearY = getValue() / 100;
                at.shear(shearX, shearY);
            }
        });
        jp.add(new JLabel("scaleX"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 200, 100) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                scaleX = getValue() / 100;
                at.scale(scaleX, scaleY);
            }
        });
        jp.add(new JLabel("scaleY"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 1, 200, 100) {
            public void run() {
                pcs.firePropertyChange("square", this, null);
                scaleY = getValue() / 100;
                at.scale(scaleX, scaleY);
            }
        });
        jp.add(new RunButton("color") {
            public void run() {
                Color c = In.getColor();
                pcs.firePropertyChange("coneHead", this, null);
                Square2d.this.setForeground(c);
            }
        });
        return jp;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        pcs.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        pcs.removePropertyChangeListener(listener);
    }

    public Object getValue() {
        return value;
    }

    public void setValue(Object newValue) {
        value = (String) newValue;
    }

    public String getAsText() {
        return null;
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String[] getTags() {
        //return new String[0];
        return null;
    }

    public void setAsText(String text) throws IllegalArgumentException {
    }

    public void paintValue(Graphics gfx, Rectangle box) {
        //To change body of implemented methods use File | Settings | File Templates.
        gfx.setClip(box);
        gfx.drawString("click to edit...", box.x + 5, box.y + 15);
    }
}

